@@ -15,20 +15,61 @@
 #include "misc.h"
 #include "html_common.hh"
 #include "styleengine.hh"
+#include "web.hh"
+#include "capi.h"
 
 using namespace lout::misc;
 using namespace dw::core::style;
 
-StyleEngine::StyleEngine (dw::core::Layout *layout) {
+/**
+ * Signal handler for "delete": This handles the case when an instance
+ * of StyleImage is deleted, possibly when the cache client is still
+ * active.
+ *
+ * \todo Not neccessary for dw::Image? (dw::Image also implements
+ * lout::signal::ObservedObject.)
+ */
+class StyleImageDeletionReceiver:
+   public lout::signal::ObservedObject::DeletionReceiver
+{
+   int clientKey;
+
+public:
+   StyleImageDeletionReceiver (int clientKey);
+   ~StyleImageDeletionReceiver ();
+
+   void deleted (lout::signal::ObservedObject *object);
+};
+
+StyleImageDeletionReceiver::StyleImageDeletionReceiver (int clientKey)
+{
+   this->clientKey = clientKey;
+}
+
+StyleImageDeletionReceiver::~StyleImageDeletionReceiver ()
+{
+}
+
+void StyleImageDeletionReceiver::deleted (lout::signal::ObservedObject *object)
+{
+   a_Capi_stop_client (clientKey, 0);
+   delete this;
+}
+
+// ----------------------------------------------------------------------
+
+StyleEngine::StyleEngine (dw::core::Layout *layout,
+                          const DilloUrl *pageUrl, const DilloUrl *baseUrl) {
    StyleAttrs style_attrs;
    FontAttrs font_attrs;
 
    doctree = new Doctree ();
    stack = new lout::misc::SimpleVector <Node> (1);
    cssContext = new CssContext ();
-   buildUserAgentStyle ();
    buildUserStyle ();
    this->layout = layout;
+   this->pageUrl = pageUrl ? a_Url_dup(pageUrl) : NULL;
+   this->baseUrl = baseUrl ? a_Url_dup(baseUrl) : NULL;
    importDepth = 0;
 
    stackPush ();
@@ -61,6 +102,9 @@ StyleEngine::~StyleEngine () {
    stackPop (); // dummy node on the bottom of the stack
    assert (stack->size () == 0);
 
+   a_Url_free(pageUrl);
+   a_Url_free(baseUrl);
+
    delete stack;
    delete doctree;
    delete cssContext;
@@ -92,8 +136,8 @@ void StyleEngine::stackPop () {
 /**
  * \brief tell the styleEngine that a new html element has started.
  */
-void StyleEngine::startElement (int element) {
-   style (); // ensure that style of current node is computed
+void StyleEngine::startElement (int element, BrowserWindow *bw) {
+   style (bw); // ensure that style of current node is computed
 
    stackPush ();
    Node *n = stack->getLastRef ();
@@ -103,8 +147,8 @@ void StyleEngine::startElement (int element) {
    n->doctreeNode = dn;
 }
 
-void StyleEngine::startElement (const char *tagname) {
-   startElement (a_Html_tag_index (tagname));
+void StyleEngine::startElement (const char *tagname, BrowserWindow *bw) {
+   startElement (a_Html_tag_index (tagname), bw);
 }
 
 void StyleEngine::setId (const char *id) {
@@ -152,7 +196,7 @@ void StyleEngine::setStyle (const char *styleAttr) {
       n->styleAttrProperties = new CssPropertyList (true);
       n->styleAttrPropertiesImportant = new CssPropertyList (true);
 
-      CssParser::parseDeclarationBlock (styleAttr, strlen (styleAttr),
+      CssParser::parseDeclarationBlock (baseUrl, styleAttr, strlen (styleAttr),
                                         n->styleAttrProperties,
                                         n->styleAttrPropertiesImportant);
    }
@@ -207,6 +251,26 @@ dw::core::style::Color *StyleEngine::backgroundColor () {
    return NULL;
 }
 
+dw::core::style::StyleImage *StyleEngine::backgroundImage
+   (dw::core::style::BackgroundRepeat *bgRepeat,
+    dw::core::style::BackgroundAttachment *bgAttachment,
+    dw::core::style::Length *bgPositionX,
+    dw::core::style::Length *bgPositionY) {
+   for (int i = 1; i < stack->size (); i++) {
+      Node *n = stack->getRef (i);
+
+      if (n->style && n->style->backgroundImage) {
+         *bgRepeat = n->style->backgroundRepeat;
+         *bgAttachment = n->style->backgroundAttachment;
+         *bgPositionX = n->style->backgroundPositionX;
+         *bgPositionY = n->style->backgroundPositionY;
+         return n->style->backgroundImage;
+      }
+   }
+
+   return NULL;
+}
+
 /**
  * \brief set the CSS pseudo class :link.
  */
@@ -238,6 +302,16 @@ void StyleEngine::preprocessAttrs (dw::core::style::StyleAttrs *attrs) {
    if (stack->getRef (stack->size () - 2)->inheritBackgroundColor) {
       attrs->backgroundColor =
          stack->getRef (stack->size () - 2)->style->backgroundColor;
+      attrs->backgroundImage =
+         stack->getRef (stack->size () - 2)->style->backgroundImage;
+      attrs->backgroundRepeat =
+         stack->getRef (stack->size () - 2)->style->backgroundRepeat;
+      attrs->backgroundAttachment =
+         stack->getRef (stack->size () - 2)->style->backgroundAttachment;
+      attrs->backgroundPositionX =
+         stack->getRef (stack->size () - 2)->style->backgroundPositionX;
+      attrs->backgroundPositionY =
+         stack->getRef (stack->size () - 2)->style->backgroundPositionY;
 
       attrs->valign = stack->getRef (stack->size () - 2)->style->valign;
    }
@@ -281,11 +355,13 @@ void StyleEngine::postprocessAttrs (dw::core::style::StyleAttrs *attrs) {
 /**
  * \brief Make changes to StyleAttrs attrs according to CssPropertyList props.
  */
-void StyleEngine::apply (int i, StyleAttrs *attrs, CssPropertyList *props) {
+void StyleEngine::apply (int i, StyleAttrs *attrs, CssPropertyList *props,
+                         BrowserWindow *bw) {
    FontAttrs fontAttrs = *attrs->font;
    Font *parentFont = stack->get (i - 1).style->font;
    char *c, *fontName;
    int lineHeight;
+   DilloUrl *imgUrl = NULL;
 
    /* Determine font first so it can be used to resolve relative lengths. */
    for (int i = 0; i < props->size (); i++) {
@@ -441,12 +517,29 @@ void StyleEngine::apply (int i, StyleAttrs *attrs, CssPropertyList *props) {
 
       switch (p->name) {
          /* \todo missing cases */
+         case CSS_PROPERTY_BACKGROUND_ATTACHMENT:
+            attrs->backgroundAttachment =
+               (BackgroundAttachment) p->value.intVal;
+            break;
          case CSS_PROPERTY_BACKGROUND_COLOR:
             if (prefs.allow_white_bg || p->value.intVal != 0xffffff)
                attrs->backgroundColor = Color::create(layout, p->value.intVal);
             else
-               //attrs->backgroundColor = Color::create(layout, 0xdcd1ba);
-               attrs->backgroundColor = Color::create(layout, 0xe0e0a3);
+               attrs->backgroundColor =
+                  Color::create(layout, prefs.white_bg_replacement);
+            break;
+         case CSS_PROPERTY_BACKGROUND_IMAGE:
+            // p->value.strVal should be absolute, so baseUrl is not needed
+            imgUrl = a_Url_new (p->value.strVal, NULL);
+            break;
+         case CSS_PROPERTY_BACKGROUND_POSITION:
+            computeLength (&attrs->backgroundPositionX, p->value.posVal->posX,
+                           attrs->font);
+            computeLength (&attrs->backgroundPositionY, p->value.posVal->posY,
+                           attrs->font);
+            break;
+         case CSS_PROPERTY_BACKGROUND_REPEAT:
+            attrs->backgroundRepeat = (BackgroundRepeat) p->value.intVal;
             break;
          case CSS_PROPERTY_BORDER_COLLAPSE:
             attrs->borderCollapse = (BorderCollapse) p->value.intVal;
@@ -615,6 +708,31 @@ void StyleEngine::apply (int i, StyleAttrs *attrs, CssPropertyList *props) {
       }
    }
 
+   if (imgUrl && prefs.load_background_images && attrs->display != DISPLAY_NONE)
+   {
+      attrs->backgroundImage = StyleImage::create();
+      DilloImage *image =
+         a_Image_new(layout,
+            (void*)attrs->backgroundImage
+            ->getMainImgRenderer(),
+            0xffffff);
+
+      // we use the pageUrl as requester to prevent cross
+      // domain requests as specified in domainrc
+      DilloWeb *web = a_Web_new(bw, imgUrl, pageUrl);
+      web->Image = image;
+      a_Image_ref(image);
+      web->flags |= WEB_Image;
+
+      int clientKey;
+      if ((clientKey = a_Capi_open_url(web, NULL, NULL)) != 0) {
+                  a_Bw_add_client(bw, clientKey, 0);
+                  a_Bw_add_url(bw, imgUrl);
+                  attrs->backgroundImage->connectDeletion
+                     (new StyleImageDeletionReceiver (clientKey));
+      }
+   }
+   a_Url_free (imgUrl);
 }
 
 /**
@@ -706,9 +824,9 @@ void StyleEngine::computeBorderWidth (int *dest, CssProperty *p,
  * A normal style might have backgroundColor == NULL to indicate a transparent
  * background. This method ensures that backgroundColor is set.
  */
-Style * StyleEngine::backgroundStyle () {
+Style * StyleEngine::backgroundStyle (BrowserWindow *bw) {
    if (!stack->getRef (stack->size () - 1)->backgroundStyle) {
-      StyleAttrs attrs = *style ();
+      StyleAttrs attrs = *style (bw);
 
       for (int i = stack->size () - 1; i >= 0 && ! attrs.backgroundColor; i--)
          attrs.backgroundColor = stack->getRef (i)->style->backgroundColor;
@@ -725,7 +843,7 @@ Style * StyleEngine::backgroundStyle () {
  * HTML elements and the nonCssProperties that have been set.
  * This method is private. Call style() to get a current style object.
  */
-Style * StyleEngine::style0 (int i) {
+Style * StyleEngine::style0 (int i, BrowserWindow *bw) {
    CssPropertyList props, *styleAttrProperties, *styleAttrPropertiesImportant;
    CssPropertyList *nonCssProperties;
    // get previous style from the stack
@@ -753,7 +871,7 @@ Style * StyleEngine::style0 (int i) {
                       nonCssProperties);
 
    // apply style
-   apply (i, &attrs, &props);
+   apply (i, &attrs, &props, bw);
 
    postprocessAttrs (&attrs);
 
@@ -762,14 +880,20 @@ Style * StyleEngine::style0 (int i) {
    return stack->getRef (i)->style;
 }
 
-Style * StyleEngine::wordStyle0 () {
-   StyleAttrs attrs = *style ();
+Style * StyleEngine::wordStyle0 (BrowserWindow *bw) {
+   StyleAttrs attrs = *style (bw);
    attrs.resetValues ();
 
-   if (stack->getRef (stack->size() - 1)->inheritBackgroundColor)
-      attrs.backgroundColor = style ()->backgroundColor;
+   if (stack->getRef (stack->size() - 1)->inheritBackgroundColor) {
+      attrs.backgroundColor = style (bw)->backgroundColor;
+      attrs.backgroundImage = style (bw)->backgroundImage;
+      attrs.backgroundRepeat = style (bw)->backgroundRepeat;
+      attrs.backgroundAttachment = style (bw)->backgroundAttachment;
+      attrs.backgroundPositionX = style (bw)->backgroundPositionX;
+      attrs.backgroundPositionY = style (bw)->backgroundPositionY;
+   }
 
-   attrs.valign = style ()->valign;
+   attrs.valign = style (bw)->valign;
 
    stack->getRef(stack->size() - 1)->wordStyle = Style::create(&attrs);
    return stack->getRef (stack->size () - 1)->wordStyle;
@@ -782,7 +906,7 @@ Style * StyleEngine::wordStyle0 () {
  * and thereby after the HTML-element has been opened.
  * Note that restyle() does not change any styles in the widget tree.
  */
-void StyleEngine::restyle () {
+void StyleEngine::restyle (BrowserWindow *bw) {
    for (int i = 1; i < stack->size (); i++) {
       Node *n = stack->getRef (i);
       if (n->style) {
@@ -798,7 +922,7 @@ void StyleEngine::restyle () {
          n->backgroundStyle = NULL;
       }
 
-      style0 (i);
+      style0 (i, bw);
    }
 }
 
@@ -820,7 +944,7 @@ void StyleEngine::parse (DilloHtml *html, DilloUrl *url, const char *buf,
  * The user agent style defines how dillo renders HTML in the absence of
  * author or user styles.
  */
-void StyleEngine::buildUserAgentStyle () {
+void StyleEngine::init () {
    const char *cssBuf =
       "body  {margin: 5px}"
       "big {font-size: 1.17em}"
@@ -830,7 +954,9 @@ void StyleEngine::buildUserAgentStyle () {
       ":link {color: blue; text-decoration: underline; cursor: pointer}"
       ":visited {color: #800080; text-decoration: underline; cursor: pointer}"
       "h1, h2, h3, h4, h5, h6, b, strong {font-weight: bolder}"
-      "address, center, div, h1, h2, h3, h4, h5, h6, ol, p, ul, pre {display: block}"
+      "address, article, aside, center, div, figure, figcaption, footer,"
+      " h1, h2, h3, h4, h5, h6, header, nav, ol, p, pre, section, ul"
+      " {display: block}"
       "i, em, cite, address, var {font-style: italic}"
       ":link img, :visited img {border: 1px solid}"
       "frameset, ul, ol, dir {margin-left: 40px}"
@@ -839,6 +965,7 @@ void StyleEngine::buildUserAgentStyle () {
        * look better like this.
        */
       "p {margin: 0.5em 0}"
+      "figure {margin: 1em 40px}"
       "h1 {font-size: 2em; margin-top: .67em; margin-bottom: 0}"
       "h2 {font-size: 1.5em; margin-top: .75em; margin-bottom: 0}"
       "h3 {font-size: 1.17em; margin-top: .83em; margin-bottom: 0}"
@@ -853,11 +980,15 @@ void StyleEngine::buildUserAgentStyle () {
       "ul ul {list-style-type: circle}"
       "ul ul ul {list-style-type: square}"
       "ul ul ul ul {list-style-type: disc}"
-      "u {text-decoration: underline}"
+      "ins, u {text-decoration: underline}"
       "small, sub, sup {font-size: 0.83em}"
       "sub {vertical-align: sub}"
       "sup {vertical-align: super}"
       "s, strike, del {text-decoration: line-through}"
+      /* HTML5 spec notes that mark styling "is just a suggestion and can be
+       * changed based on implementation feedback"
+       */
+      "mark {background: yellow; color: black;}"
       "table {border-spacing: 2px}"
       "td, th {padding: 2px}"
       "thead, tbody, tfoot {vertical-align: middle}"
@@ -870,7 +1001,8 @@ void StyleEngine::buildUserAgentStyle () {
        */
       "table, caption {font-size: medium; font-weight: normal}";
 
-   CssParser::parse (NULL, NULL, cssContext, cssBuf, strlen (cssBuf),
+   CssContext context;
+   CssParser::parse (NULL, NULL, &context, cssBuf, strlen (cssBuf),
                      CSS_ORIGIN_USER_AGENT);
 }
 
